from collections.abc import Iterable
from typing import Protocol, Sized

from matchescu.typing import EntityReferenceIdentifier, EntityReference


class IdTable(Iterable, Sized, Protocol):
    def get(self, ref_id: EntityReferenceIdentifier) -> EntityReference:
        pass

    def get_by_source(self, source: str) -> Iterable[EntityReference]:
        pass

    def put(self, ref: EntityReference) -> None:
        pass



from functools import partial
from typing import Iterable

from matchescu.reference_store._exceptions import EntityReferenceNotFound
from matchescu.typing import EntityReference, EntityReferenceIdentifier


class InMemoryIdTable(object):
    def __init__(self):
        self._id_table = {}

    def __len__(self) -> int:
        return len(self._id_table)

    def __iter__(self) -> Iterable[EntityReference]:
        return iter(self._id_table.values())

    def put(self, ref: EntityReference) -> None:
        if ref is None:
            return
        self._id_table[ref.id] = ref

    def get(self, ref_id: EntityReferenceIdentifier) -> EntityReference:
        if ref_id not in self._id_table:
            raise EntityReferenceNotFound(ref_id)
        return self._id_table[ref_id]

    @staticmethod
    def __has_source(identifier: EntityReferenceIdentifier, source: str) -> bool:
        return identifier.source == source

    def get_by_source(self, source: str) -> Iterable[EntityReference]:
        has_source = partial(self.__has_source, source=source)
        ids_with_source = filter(has_source, self._id_table.keys())
        return map(self._id_table.get, ids_with_source)



from matchescu.typing import EntityReferenceIdentifier


class EntityReferenceNotFound(Exception):
    def __init__(self, identifier: EntityReferenceIdentifier) -> None:
        super().__init__(
            f"Entity reference with label '{identifier.label}' from source '{identifier.source}' not found"
        )



from collections.abc import Iterator

from matchescu.typing import EntityReference, EntityReferenceIdentifier


class InMemoryComparisonSpace:
    def __init__(self):
        self.__data = {}

    def put(self, ref1: EntityReference, ref2: EntityReference) -> None:
        key = (ref1.id, ref2.id)
        val = self.__data.get(key, 0)
        self.__data[key] = val + 1

    def __len__(self) -> int:
        return len(self.__data)

    def __iter__(
        self,
    ) -> Iterator[tuple[EntityReferenceIdentifier, EntityReferenceIdentifier]]:
        return iter(self.__data.keys())
